-------------------------------------------------------------------------------
--! @file       CipherCore.vhd
--! @brief      Cipher core template module
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @copyright  Copyright (c) 2015 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------
--3/16/2016
library ieee;
use ieee.std_logic_1164.ALL;

entity CipherCore is
    generic (
        --! Reset behavior
        G_ASYNC_RSTN    : boolean := False; --! Async active low reset
        --! Block size (bits)
        G_DBLK_SIZE     : integer := 128;   --! Data
        G_KEY_SIZE      : integer := 128;   --! Key
        G_TAG_SIZE      : integer := 128;   --! Tag
        --! The number of bits required to hold block size expressed in
        --! bytes = log2_ceil(G_DBLK_SIZE/8)
        G_LBS_BYTES     : integer := 4;
		  G_MAX_LEN			: integer := 32 --! 32 for 2^32-1, 11 for 2^11 - 1
    );
    port (
        --! Global
        clk             : in  std_logic;
        rst             : in  std_logic;
        --! PreProcessor (data)
        key             : in  std_logic_vector(G_KEY_SIZE       -1 downto 0);
        bdi             : in  std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        --! PreProcessor (controls)
        key_ready       : out std_logic;
        key_valid       : in  std_logic;
        key_update      : in  std_logic;
        decrypt         : in  std_logic;
        bdi_ready       : out std_logic;
        bdi_valid       : in  std_logic;
        bdi_type        : in  std_logic_vector(3                -1 downto 0);
        bdi_partial     : in  std_logic;
        bdi_eot         : in  std_logic;
        bdi_eoi         : in  std_logic;
        bdi_size        : in  std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        bdi_valid_bytes : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        bdi_pad_loc     : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        --! PostProcessor
        bdo             : out std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        bdo_valid       : out std_logic;
        bdo_ready       : in  std_logic;
        bdo_size        : out std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        msg_auth_done   : out std_logic;
        msg_auth_valid  : out std_logic
    );
end entity CipherCore;

architecture structure of CipherCore is

--CONSTANT rand_seed : std_logic_vector(127 downto 0):=x"0123456789abcdef0123456789abcdef"; --only for masking
SIGNAL auth_sum_init : STD_LOGIC;
SIGNAL load_key, load_npub, load_tag, load_sum, load_auth: STD_LOGIC;
SIGNAL tag_valid: STD_LOGIC;

SIGNAL decrypt_reg, decrypt_init: STD_LOGIC;
SIGNAL en : std_logic;
SIGNAL block_counter_init, en_block_counter : std_logic;
SIGNAL Ek_sel, bdo_sel: std_logic_vector(1 downto 0);
SIGNAL sel_data : std_logic;
SIGNAL Ek_start, Ek_done : std_logic;
SIGNAL tag_mode : std_logic;
signal null_length : std_logic;

begin

datapath: entity work.Datapath(structural)
		 GENERIC MAP(
		  G_DBLK_SIZE         => G_DBLK_SIZE      ,
        G_KEY_SIZE          => G_KEY_SIZE       ,
        G_TAG_SIZE          => G_TAG_SIZE       ,
        G_LBS_BYTES         => G_LBS_BYTES      ,
		  G_MAX_LEN    		=> G_MAX_LEN
		  )
       PORT MAP(

		clk => clk,
		--rst => rst,
		en => en,
	 
		--rand_seed => rand_seed,  -- used only in masked version
		bdi => bdi,
		bdo => bdo,
		key_in => key,

		bdi_type => bdi_type,
		bdi_size => bdi_size,
		bdi_eot => bdi_eot,
		bdi_valid_bytes => bdi_valid_bytes,
		
		decrypt => decrypt,
		decrypt_init => decrypt_init,
		decrypt_reg_out => decrypt_reg,
		load_key	=> load_key,
		load_npub => load_npub,
		block_counter_init => block_counter_init,
		en_block_counter => en_block_counter,
		auth_sum_init => auth_sum_init,
		load_sum => load_sum,
		load_auth => load_auth,
		load_tag => load_tag,
		tag_valid => tag_valid,
		tag_mode => tag_mode,
		null_length => null_length,

		Ek_sel => Ek_sel,
		bdo_sel => bdo_sel,
		Ek_start => Ek_start,
		Ek_done => Ek_done
); 

controller: entity work.Controller(behavioral)
       PORT MAP(
		      
				clk => clk,
				reset => rst,
				en => en,

				bdi_type => bdi_type,
				bdi_size => bdi_size,
				bdi_eot => bdi_eot,
				bdi_eoi => bdi_eoi,
				decrypt => decrypt_reg,
				decrypt_init => decrypt_init,
				load_key	=> load_key,
				load_npub => load_npub,
				block_counter_init => block_counter_init,
				en_block_counter => en_block_counter,
				auth_sum_init => auth_sum_init,
				load_sum => load_sum,
				load_auth => load_auth,
				load_tag => load_tag,
				tag_valid => tag_valid,
				tag_mode => tag_mode,
				null_length => null_length,
				
				Ek_sel => Ek_sel,
				bdo_sel => bdo_sel,
				Ek_start => Ek_start,
				Ek_done => Ek_done,
				
				key_ready => key_ready,
				key_valid => key_valid,
				key_update => key_update,
				
				bdi_ready => bdi_ready,
				bdi_valid => bdi_valid,
				bdo_ready => bdo_ready,
				bdo_valid => bdo_valid,
				msg_auth_valid => msg_auth_valid,
				msg_auth_done => msg_auth_done
								
				);

end structure;